@@ -3,11 +3,11 @@
 ```
 use num::Rational64;
 use polynomial_ring::Polynomial;
-let p = Polynomial::new(vec![3, 1, 4, 1, 5].into_iter().map(|x| Rational64::from_integer(x)).collect());
-let q = Polynomial::new(vec![2, 7, 1].into_iter().map(|x| Rational64::from_integer(x)).collect());
-let mut r = p.clone();
-let d = r.division(&q);
-assert_eq!(p, d * q + r);
+let f = Polynomial::new(vec![3, 1, 4, 1, 5].into_iter().map(|x| Rational64::from_integer(x)).collect());
+let g = Polynomial::new(vec![2, 7, 1].into_iter().map(|x| Rational64::from_integer(x)).collect());
+let mut r = f.clone();
+let q = r.division(&g);
+assert_eq!(f, q * g + r);
 ```
 */
 #![cfg_attr(feature = "__internal_inject_debug", recursion_limit = "8")]
@@ -22,7 +22,7 @@ mod sealed {
 use num_traits::{One, Zero};
 use ring_algorithm::RingNormalize;
 use sealed::Sized;
-use std::ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, SubAssign};
+use std::ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Sub, SubAssign};
 mod ops;
 
 /** Polynomial ring $`R[x]`$
@@ -30,11 +30,11 @@ mod ops;
 ```
 use num::Rational64;
 use polynomial_ring::Polynomial;
-let p = Polynomial::new(vec![3, 1, 4, 1, 5].into_iter().map(|x| Rational64::from_integer(x)).collect());
-let q = Polynomial::new(vec![2, 7, 1].into_iter().map(|x| Rational64::from_integer(x)).collect());
-let mut r = p.clone();
-let d = r.division(&q);
-assert_eq!(p, d * q + r);
+let f = Polynomial::new(vec![3, 1, 4, 1, 5].into_iter().map(|x| Rational64::from_integer(x)).collect());
+let g = Polynomial::new(vec![2, 7, 1].into_iter().map(|x| Rational64::from_integer(x)).collect());
+let mut r = f.clone();
+let q = r.division(&g);
+assert_eq!(f, q * g + r);
 ```
 */
 #[derive(Clone, Debug, PartialEq, Eq, Default)]
@@ -363,6 +363,140 @@ impl<R: Sized> Polynomial<R> {
         self.coef.iter_mut().for_each(|c| *c *= alpha);
         self.trim_zero();
     }
+    /** pseudo division
+
+    Let $`R`$ be an [integral domain](https://en.wikipedia.org/wiki/Integral_domain).
+    Let $`f, g \in R[x]`$, where $`g \neq 0`$.
+    This function calculate $`s \in R`$, $`q, r \in R[x]`$ s.t. $`sf=qg+r`$,
+    where $`r=0`$ or $`\deg(r)<\deg(g)`$.
+    ```
+    use polynomial_ring::{polynomial, Polynomial};
+    let f = polynomial![1, 3, 1]; // 1+3x+x^2 ∈ Z[x]
+    let g = polynomial![5, 2]; // 5+2x ∈ Z[x]
+    let mut r = f.clone();
+    let (s, q) = r.pseudo_division(&g);
+    assert_eq!(f * s, q * g + r);
+    let f = polynomial![1, -1, -1, 1]; // 1-x-x^2+x^3 ∈ Z[x]
+    let g = polynomial![1, 2]; // 1+2x ∈ Z[x]
+    let mut r = f.clone();
+    let (s, q) = r.pseudo_division(&g);
+    assert_eq!(polynomial![s] * f, q * g + r);
+    // 1-yx-x^2+yx^3 ∈ Z[y][x]
+    let f = polynomial![polynomial![1], polynomial![0, -1], polynomial![-1], polynomial![0, 1]];
+    // -1+y^2x ∈ Z[y][x]
+    let g = polynomial![polynomial![-1], polynomial![0, 0, 1]];
+    let mut r = f.clone();
+    let (s, q) = r.pseudo_division(&g);
+    assert_eq!(f * s, q * g + r);
+    // x^3 ∈ Z[y][x]
+    let f = polynomial![polynomial![], polynomial![], polynomial![], polynomial![1]];
+    // yx ∈ Z[y][x]
+    let g = polynomial![polynomial![], polynomial![0, 1]];
+    let mut r = f.clone();
+    let (s, q) = r.pseudo_division(&g);
+    assert_eq!(f * s, q * g + r);
+    ```
+    */
+    pub fn pseudo_division(&mut self, other: &Self) -> (R, Self)
+    where
+        R: Sized + Clone + Zero + One + for<'x> AddAssign<&'x R> + for<'x> MulAssign<&'x R>,
+        for<'x> &'x R: Sub<Output = R> + Mul<Output = R>,
+    {
+        let g_deg = other.deg().expect("Division by zero");
+        let f_deg = self.deg();
+        if f_deg < other.deg() {
+            return (R::one(), Self::zero());
+        }
+        let f_deg = f_deg.unwrap();
+        assert!(f_deg >= g_deg);
+        let k = f_deg - g_deg + 1;
+        let lc = other.lc().unwrap();
+        let mut coef = vec![R::zero(); k];
+        let mut scale = R::one();
+        while self.deg() >= other.deg() {
+            let d = self.deg().unwrap() - g_deg;
+            let c = self.lc().unwrap().clone();
+            for i in 0..other.len() - 1 {
+                self.coef[i + d] = &(lc * &self.coef[i + d]) - &(&c * &other.coef[i]);
+            }
+            for i in 0..d {
+                self.coef[i] *= lc;
+            }
+            self.coef.pop(); // new deg < prev deg
+            self.trim_zero();
+            for c_i in coef.iter_mut().skip(d + 1) {
+                *c_i *= lc;
+            }
+            coef[d] = c;
+            scale *= lc;
+        }
+        (scale, Self { coef })
+    }
+    /** calculate [resultant](https://en.wikipedia.org/wiki/Resultant)
+
+    ```
+    use polynomial_ring::{polynomial, Polynomial};
+    use num_traits::{One, Zero};
+    let f = polynomial![0, 2, 0, 1]; // 2x+x^3 ∈ Z[x]
+    let g = polynomial![2, 3, 5]; // 2+3x+5x^2 ∈ Z[x]
+    let r = f.resultant(g);
+    assert_eq!(r, 164);
+    let f = polynomial![-4, 0, 0, 0, 1]; // -4+x^4 ∈ Z[x]
+    let g = polynomial![0, 2, 0, 1]; // 2x+x^3 ∈ Z[x]
+    let r = f.resultant(g); // deg(gcd(f, g)) = deg(x^2-2) = 2 ≠ 0
+    assert_eq!(r, 0);
+    let f = polynomial![polynomial![1], polynomial![0], polynomial![1]]; // 1+x^2 ∈ Z[y][x]
+    let g = polynomial![polynomial![1], polynomial![1, 2]]; // 1+(1+2y)x ∈ Z[y][x]
+    let r = f.resultant(g);
+    assert_eq!(r, polynomial![2, 4, 4]); // 2+4y+4y^2
+    let y3 = Polynomial::from_monomial(polynomial![-1], 3); // -y^3 ∈ Z[x][y]
+    let y2xy = Polynomial::from_monomial(polynomial![0, -1], 2) + &y3; // -xy^2-y^3 ∈ Z[x][y]
+    let x = polynomial![polynomial![0, 1]]; // x ∈ Z[x][y]
+    let f = polynomial![y3, polynomial![], x.clone()]; // -y^3+xz^2 ∈ Z[x][y][z]
+    let g = polynomial![y2xy, polynomial![], x]; // -xy^2-y^3+xz^2 ∈ Z[x][y][z]
+    let r = f.resultant(g);
+    assert_eq!(r, Polynomial::from_monomial(Polynomial::from_monomial(1, 4), 4)); // x^4y^4
+    ```
+    */
+    pub fn resultant(mut self, other: Self) -> R
+    where
+        R: Sized
+            + Clone
+            + Zero
+            + One
+            + for<'x> AddAssign<&'x R>
+            + for<'x> MulAssign<&'x R>
+            + Neg<Output = R>,
+        for<'x> &'x R: Sub<Output = R> + Mul<Output = R> + Div<Output = R>,
+    {
+        let f_deg = self.deg();
+        let g_deg = other.deg();
+        match (f_deg, g_deg) {
+            (Some(0), Some(0)) => R::one(),
+            (Some(0), None) => R::one(),
+            (None, Some(0)) => R::one(),
+            (None, None) => R::zero(),
+            (None, Some(_)) => R::zero(),
+            (Some(_), None) => R::zero(),
+            (Some(0), Some(m)) => ring_algorithm::power::<R>(self.lc().unwrap().clone(), m as u64),
+            (Some(n), Some(0)) => ring_algorithm::power::<R>(other.lc().unwrap().clone(), n as u64),
+            (Some(n), Some(m)) => {
+                assert!(n >= 1);
+                assert!(m >= 1);
+                let (scale, _) = self.pseudo_division(&other);
+                if let Some(l) = self.deg() {
+                    let sign = if n * m % 2 == 0 { R::one() } else { -R::one() };
+                    let mul =
+                        ring_algorithm::power::<R>(other.lc().unwrap().clone(), (n - l) as u64);
+                    let div = ring_algorithm::power::<R>(scale, m as u64);
+                    &(other.resultant(self) * sign * mul) / &div
+                } else {
+                    // g | f, gcd(f, g) = g
+                    R::zero()
+                }
+            }
+        }
+    }
 }
 
 // field
@@ -408,11 +542,11 @@ impl<K: Sized> Polynomial<K> {
     ```
     use num::Rational64;
     use polynomial_ring::Polynomial;
-    let p = Polynomial::new(vec![3, 1, 4, 1, 5].into_iter().map(|x| Rational64::from_integer(x)).collect());
-    let q = Polynomial::new(vec![2, 7, 1].into_iter().map(|x| Rational64::from_integer(x)).collect());
-    let mut r = p.clone();
-    let d = r.division(&q);
-    assert_eq!(p, d * q + r);
+    let f = Polynomial::new(vec![3, 1, 4, 1, 5].into_iter().map(|x| Rational64::from_integer(x)).collect());
+    let g = Polynomial::new(vec![2, 7, 1].into_iter().map(|x| Rational64::from_integer(x)).collect());
+    let mut r = f.clone();
+    let q = r.division(&g);
+    assert_eq!(f, q * g + r);
     ```
     */
     pub fn division(&mut self, other: &Self) -> Self
@@ -438,7 +572,7 @@ impl<K: Sized> Polynomial<K> {
         }
         Self { coef }
     }
-    /** square free
+    /** calculate [square-free polynomial](https://en.wikipedia.org/wiki/Square-free_polynomial)
 
     ```
     use polynomial_ring::{Polynomial, polynomial};
